/*++
	Copyright (c) 2008-2009 SMSC. All rights reserved.

	The information contained in this file is confidential and proprietary to
	Standard Microsystems Corporation.	No part of this file may be reproduced
	or distributed, in any form or by any means for any purpose, without the
	express written permission of Standard Microsystems Corporation.

Module Name:	
	lan9500.h
Abstract:
	This module contains structure definitons and registers of LAN9500.
Revision History:
	12/11/2008	Initial version.
Notes:
--*/

#ifndef _LAN9500_H
#define _LAN9500_H

#include <ndis.h> 
#include "nuiouser.h"
#include "version.h" 
#include "os.h"
#include "Ioctl.h"

#define ON TRUE
#define OFF FALSE

#define LAN9500_TAG ((ULONG) '0059')	  //'0059' for the Tag '9500'

// media type, we use ethernet, change if necessary
#define NIC_MEDIA_TYPE					  NdisMedium802_3

// we use Internal, change to Pci, Isa, etc. properly
#define NIC_INTERFACE_TYPE				  NdisInterfaceInternal  

//#define NDIS_ETH_TYPE_802_1Q				(0x8100)

// Timing values
#define NDIS_SLEEP_1_MS					1000
#define NDIS_SLEEP_5_MS					5000
#define NDIS_SLEEP_10_MS				10000
#define NDIS_SLEEP_100_MS				100000
#define NDIS_SLEEP_1_S					1000000
#define NDIS_SLEEP_10_S					10000000

#define NIC_2_MIN_TIMEOUT				120000		// 2 mins in ms
#define NIC_10_SEC_TIMEOUT				10000		// 10 secs in ms
#define NIC_5_SEC_TIMEOUT				5000		// 5 secs in ms
#define NIC_2_SEC_TIMEOUT				2000		// 2 secs in ms
#define NIC_1_SEC_TIMEOUT				1000		// 1 secs in ms
#define NIC_HALF_SEC_TIMEOUT			500			// 0.5 secs in ms

// Highest byte is the NIC byte plus three vendor bytes, they are normally	
// obtained from the NIC. SMSC OUI = 00-80-0F, Thera NIC ID = 95
#define DEFAULT_NIC_VENDOR_ID				0x00800f					
#define DEFAULT_NIC_DEVICE_ID				0x95				   

typedef enum {
	NIC_STATUS_SUCCESS = 0,
	NIC_STATUS_FAILURE = 1
} NIC_STATUS, *PNIC_STATUS;

/* Definitions for requested media type*/
#define MEDIA_TYPE_AUTO_FULL				0x00
#define MEDIA_TYPE_10HD_AUTO				0x01
#define MEDIA_TYPE_10FD_AUTO				0x02
#define MEDIA_TYPE_100HD_AUTO				0x03
#define MEDIA_TYPE_100FD_AUTO				0x04
#define MEDIA_TYPE_AUTO_CUSTOM				0x05
#define MEDIA_TYPE_10HD_FORCED				0x06
#define MEDIA_TYPE_10FD_FORCED				0x07
#define MEDIA_TYPE_100HD_FORCED				0x08
#define MEDIA_TYPE_100FD_FORCED				0x09

/* Receive Mask definitions */
#define ACCEPT_MULTICAST				0x0001
#define ACCEPT_BROADCAST				0x0002
#define PROMISCUOUS_MODE				0x0004
#define ACCEPT_SOURCE_ROUTING			0x0008
#define ACCEPT_ERR_PACKETS				0x0010
#define ACCEPT_ATT_MAC_FRAMES			0x0020
#define ACCEPT_MULTI_PROM				0x0040
#define TRANSMIT_ONLY					0x0080
#define ACCEPT_EXT_MAC_FRAMES			0x0100
#define EARLY_RX_ENABLE 				0x0200
#define PKT_SIZE_NOT_NEEDED 			0x0400
#define ACCEPT_SOURCE_ROUTING_SPANNING	0x0808
#define ACCEPT_ALL_MAC_FRAMES			0x0120
#define REJECT_DIRECTED 				0x1000

/* Definitions for link speed */
#define LINK_SPEED_UNKNOWN			0x0000
#define LINK_SPEED_10MBS			0x0001
#define LINK_SPEED_100MBS			0x0002

#define LINK_DUPLEX_UNKNOWN 		0x0000
#define LINK_DUPLEX_HALF			0x0001
#define LINK_DUPLEX_FULL			0x0002

/* Definitions for Auto MDIX */
#define AMDIX_DISABLE_STRAIGHT			0x00000000UL
#define AMDIX_DISABLE_CROSSOVER 		0x00000001UL
#define AMDIX_ENABLE					0x00000002UL
#define AMDIX_USE_STRAP 				0x00000003UL


// Tx MAT Cmds
#define 	TX_CMD_A_DATA_OFFSET_		(0x001F0000UL)
#define 	TX_CMD_A_FIRST_SEG_ 		(0x00002000UL)
#define 	TX_CMD_A_LAST_SEG_			(0x00001000UL)
#define 	TX_CMD_A_BUF_SIZE_			(0x000007FFUL)

#define 	TX_CMD_B_CSUM_ENABLE		(0x00004000UL)
#define 	TX_CMD_B_ADD_CRC_DISABLE_	(0x00002000UL)
#define 	TX_CMD_B_DISABLE_PADDING_	(0x00001000UL)
#define 	TX_CMD_B_PKT_BYTE_LENGTH_	(0x000007FFUL)

// Rx Sts Wd
#define	RX_STS_FF_			0x40000000UL	// Filter Fail
#define	RX_STS_FL_			0x3FFF0000UL	// Frame Length
#define RX_STS_DL_SHIFT_			16UL
#define	RX_STS_ES_			0x00008000UL	// Error Summary
#define	RX_STS_BF_			0x00002000UL	// Broadcast Frame
#define	RX_STS_LE_			0x00001000UL	// Length Error
#define	RX_STS_RF_			0x00000800UL	// Runt Frame
#define	RX_STS_MF_			0x00000400UL	// Multicast Frame
#define	RX_STS_TL_			0x00000080UL	// Frame too long
#define	RX_STS_CS_			0x00000040UL	// Collision Seen
#define	RX_STS_FT_			0x00000020UL	// Frame Type
#define	RX_STS_RW_			0x00000010UL	// Receive Watchdog
#define	RX_STS_ME_			0x00000008UL	// Mii Error
#define RX_STS_DB_			0x00000004UL	// Dribbling
#define RX_STS_CRC_			0x00000002UL	// CRC Error
#define RX_STS_ERR_MASK_	0x000088DAUL
#define RX_STS_ERR_DROP		(RX_STS_CRC_ | RX_STS_ME_ | RX_STS_RW_ | RX_STS_CS_ | RX_STS_RF_)

// SCSRs
#define ETH_ID_REV						(0x00UL)
	#define ETH_ID_REV_CHIP_ID_MASK_		(0xffff0000UL)
	#define ETH_ID_REV_CHIP_REV_MASK_		(0x0000ffffUL)
	#define ETH_ID_REV_CHIP_ID_9500_			(0x9500UL)	// Thera 180
	#define ETH_ID_REV_CHIP_ID_9505_			(0x9505UL)	// Thera 180, no MEF
	#define ETH_ID_REV_CHIP_ID_9E00_			(0x9E00UL)	// Thera 130
	#define ETH_ID_REV_CHIP_ID_9E01_			(0x9E01UL)	// Thera 130, no MEF
	#define ETH_ID_REV_CHIP_ID_EC00_			(0xEC00UL)	// Andros
	#define ETH_ID_REV_CHIP_ID_9E08_            (0x9E08UL)  // Thera Auto
    #define ETH_ID_REV_CHIP_ID_9E09_            (0x9E09UL)  
    #define ETH_ID_REV_CHIP_ID_9730_            (0x9730UL)  
    #define ETH_ID_REV_CHIP_ID_9735_            (0x9735UL)  
    #define ETH_ID_REV_CHIP_ID_9530_            (0x9530UL)  
    #define ETH_ID_REV_CHIP_ID_9535_            (0x9535UL)  
	#define GetChipIdFromID_REV(dwReg)			((dwReg & ETH_ID_REV_CHIP_ID_MASK_) >> 16)
	#define GetChiprevFromID_REV(dwReg) 		(dwReg & ETH_ID_REV_CHIP_REV_MASK_)

#define	ETH_FPGA_REV					(0x04UL)

#define ETH_INT_STS 					(0x08UL)
	#define	ETH_INT_STS_MACRTO_				(0x00040000UL)
	#define	ETH_INT_STS_TX_STOP_			(0x00020000UL)
	#define	ETH_INT_STS_RX_STOP_			(0x00010000UL)
	#define	ETH_INT_STS_PHY_INT_			(0x00008000UL)
	#define	ETH_INT_STS_TXE_				(0x00004000UL)
	#define	ETH_INT_STS_TDFU_				(0x00002000UL)
	#define	ETH_INT_STS_TDFO_				(0x00001000UL)
	#define	ETH_INT_STS_RXDF_				(0x00000800UL)
	#define	ETH_INT_STS_GPIOS_				(0x000007FFUL)

#define ETH_RX_CFG						(0x0CUL)
	#define ETH_RX_FIFO_FLUSH_				(0x00000001UL)

#define ETH_TX_CFG						(0x10UL)
	#define ETH_TX_ON_						(0x00000004UL)
	#define ETH_TX_STOP_					(0x00000002UL)
	#define ETH_TX_FIFO_FLUSH_				(0x00000001UL)

#define ETH_HW_CFG						(0x14UL)
	#define ETH_HW_CFG_SMDET_STS_           (0x00040000UL)
    	#define ETH_HW_CFG_SMDET_EN_            (0x00020000UL)
    	#define ETH_HW_CFG_EEM_                 (0x00010000UL)
    	#define ETH_HW_CFG_RST_PROTECT_         (0x00008000UL)
	#define ETH_HW_CFG_PHY_BOOST_			(0x00006000UL)
		#define ETH_HW_CFG_PHY_BOOST_NORMAL_	(0x00000000UL)
		#define ETH_HW_CFG_PHY_BOOST_PLUS_4_	(0x00002000UL)
		#define ETH_HW_CFG_PHY_BOOST_PLUS_8_	(0x00004000UL)
		#define ETH_HW_CFG_PHY_BOOST_PLUS_12_	(0x00006000UL)

	#define ETH_HW_CFG_BIR_ 				(0x00001000UL)
	#define ETH_HW_CFG_LEDB_				(0x00000800UL)
	#define ETH_HW_CFG_RXDOFF_				(0x00000600UL)
	#define ETH_HW_CFG_SBP_ 				(0x00000100UL)
	#define ETH_HW_CFG_IME_ 				(0x00000080UL)
	#define ETH_HW_CFG_DRP_ 				(0x00000040UL)
	#define ETH_HW_CFG_MEF_ 				(0x00000020UL)
	#define ETH_HW_CFG_LRST_				(0x00000008UL)
	#define ETH_HW_CFG_PSEL_				(0x00000004UL)
	#define ETH_HW_CFG_BCE_ 				(0x00000002UL)
	#define ETH_HW_CFG_SRST_				(0x00000001UL)

#define ETH_RX_FIFO_INF 				(0x18UL)

#define ETH_TX_FIFO_INF 				(0x1CUL)

#define ETH_PM_CTRL						(0x20UL)
	#define PM_CTL_RES_CLR_WKP_MASK			(0x00000300UL)
	#define PM_CTL_RES_CLR_WKP_STS_			(0x00000200UL)
	#define PM_CTL_RES_CLR_WKP_EN_			(0x00000100UL)
	#define PM_CTL_DEV_RDY_					(0x00000080UL)
	#define	PM_CTL_SUS_MODE_					(0x00000060UL)
		#define	PM_CTL_SUS_MODE_0				(0x00000000UL)
		#define	PM_CTL_SUS_MODE_1				(0x00000020UL)
		#define	PM_CTL_SUS_MODE_2				(0x00000040UL)
		#define	PM_CTL_SUS_MODE_3				(0x00000060UL)
	#define PM_CTL_PHY_RST_					(0x00000010UL)
	#define PM_CTL_WOL_EN_					(0x00000008UL)
	#define PM_CTL_ED_EN_					(0x00000004UL)
	#define PM_CTL_WUPS_					(0x00000003UL)
		#define PM_CTL_WUPS_NO_					(0x00000000UL)
		#define PM_CTL_WUPS_ED_					(0x00000001UL)
		#define PM_CTL_WUPS_WOL_				(0x00000002UL)
		#define PM_CTL_WUPS_MULTI_				(0x00000003UL)

#define ETH_LED_GPIO_CFG				(0x24UL)
	#define ETH_LED_GPIO_CFG_LED_SEL_		(0x80000000UL)
	#define ETH_LED_GPIO_CFG_GPCTL_10_		(0x03000000UL)
	#define ETH_LED_GPIO_CFG_GPCTL_10_SH				24
	#define ETH_LED_GPIO_CFG_GPCTL_09_		(0x00300000UL)
	#define ETH_LED_GPIO_CFG_GPCTL_09_SH				20
	#define ETH_LED_GPIO_CFG_GPCTL_08_		(0x00030000UL)
	#define ETH_LED_GPIO_CFG_GPCTL_08_SH				16
	
	#define ETH_LED_GPIO_CFG_GPCTL_DIAG2_	(0x3UL)
	#define ETH_LED_GPIO_CFG_GPCTL_DIAG1_	(0x2UL)
	#define ETH_LED_GPIO_CFG_GPCTL_LED_ 	(0x1UL)
	#define ETH_LED_GPIO_CFG_GPCTL_GPIO_	(0x0UL)

	#define ETH_LED_GPIO_CFG_GPBUF_			(0x00000700UL)
		#define ETH_LED_GPIO_CFG_GPBUF_10_		(0x00000400UL)
		#define ETH_LED_GPIO_CFG_GPBUF_09_		(0x00000200UL)
		#define ETH_LED_GPIO_CFG_GPBUF_08_		(0x00000100UL)
	#define ETH_LED_GPIO_CFG_GPDIR_			(0x00000070UL)
		#define ETH_LED_GPIO_CFG_GPDIR_10_		(0x00000040UL)
		#define ETH_LED_GPIO_CFG_GPDIR_09_		(0x00000020UL)
		#define ETH_LED_GPIO_CFG_GPDIR_08_		(0x00000010UL)
	#define ETH_LED_GPIO_CFG_GPDATA_		(0x00000070UL)
		#define ETH_LED_GPIO_CFG_GPDATA_10_		(0x00000004UL)
		#define ETH_LED_GPIO_CFG_GPDATA_09_		(0x00000002UL)
		#define ETH_LED_GPIO_CFG_GPDATA_08_		(0x00000001UL)


#define ETH_GPIO_CFG					(0x28UL)
	#define ETH_GPIO_CFG_GPEN_				(0xFF000000UL)
		#define ETH_GPIO_CFG_GPO0_DISABLED_ 		  (0x01000000UL)
	#define ETH_GPIO_CFG_GPTYPE 			(0x00FF0000UL)
		#define ETH_GPIO_CFG_GPO0_TYPE			(0x00010000UL)
	#define ETH_GPIO_CFG_GPDIR_ 			(0x0000FF00UL)
		#define ETH_GPIO_CFG_GPO0_DIR_			(0x00000100UL)
	#define ETH_GPIO_CFG_GPDATA_			(0x000000FFUL)
		#define ETH_GPIO_CFG_GPO0_DATA_ 		(0x00000001UL)


#define ETH_AFC_CFG						(0x2CUL)
	#define ETH_AFC_CFG_HI_					(0x00ff0000UL)
	#define ETH_AFC_CFG_LO_					(0x0000ff00UL)
	#define ETH_AFC_CFG_BD_					(0x000000f0UL)
	#define ETH_AFC_CFG_FCM_				(0x00000008UL)
	#define ETH_AFC_CFG_FCB_				(0x00000004UL)
	#define ETH_AFC_CFG_FCU_				(0x00000002UL)
	#define ETH_AFC_CFG_FCA_				(0x00000001UL)
	#define ETH_AFC_CFG_DEFAULT				(0x00f830A1UL)	// Hi watermark = 15.5Kb (~10 mtu pkts)
															// low watermark = 3k (~2 mtu pkts)
															// backpressure duration = ~ 350us
															// Apply FC on any frame.

#define ETH_E2P_CMD						(0x30UL)
	#define	ETH_E2P_CMD_BUSY_				(0x80000000UL)
	#define	ETH_E2P_CMD_MASK_				(0x70000000UL)
	#define	ETH_E2P_CMD_READ_				(0x00000000UL)
	#define	ETH_E2P_CMD_EWDS_				(0x10000000UL)
	#define	ETH_E2P_CMD_EWEN_				(0x20000000UL)
	#define	ETH_E2P_CMD_WRITE_				(0x30000000UL)
	#define	ETH_E2P_CMD_WRAL_				(0x40000000UL)
	#define	ETH_E2P_CMD_ERASE_				(0x50000000UL)
	#define	ETH_E2P_CMD_ERAL_				(0x60000000UL)
	#define	ETH_E2P_CMD_RELOAD_				(0x70000000UL)
	#define	ETH_E2P_CMD_TIMEOUT_			(0x00000400UL)
	#define	ETH_E2P_CMD_LOADED_				(0x00000200UL)
	#define	ETH_E2P_CMD_ADDR_				(0x000001FFUL)

#define ETH_E2P_DATA					(0x34UL)
	#define	ETH_E2P_DATA_MASK_				(0x000000FFUL)

#define ETH_BURST_CAP					(0x38UL)

#define ETH_STRAP_DEBUG					(0x3CUL)
	 #define ETH_STRAP_DEBUG_AMDIX_EN_	 0x00000010UL

#define ETH_DP_SEL						(0x40UL)
	#define     ETH_DP_SEL_DPRDY_                (0x80000000UL)
	#define     ETH_DP_SEL_RX_RAM_BITS_          (0x00000006UL)
	#define     ETH_DP_SEL_RX_TLI_RAM_           (0x00000006UL)
	#define     ETH_DP_SEL_TX_TLI_RAM_           (0x00000004UL)
	#define     ETH_DP_SEL_TX_EEP_RAM_           (0x00000002UL)
	#define     ETH_DP_SEL_TX_FCT_RAM_           (0x00000000UL)
	#define     ETH_DP_SEL_TESTEN_               (0x00000001UL)


#define ETH_DP_CMD						(0x44UL)
	#define     ETH_DP_CMD_DP_WR                 (0x00000001UL)
	
#define ETH_DP_ADDR						(0x48UL)

#define ETH_DP_DATA0					(0x4CUL)

#define ETH_DP_DATA1					(0x50UL)

#define ETH_RAM_BIST0					(0x54UL)

#define ETH_RAM_BIST1					(0x58UL)

#define ETH_RAM_BIST2					(0x5CUL)

#define ETH_RAM_BIST3					(0x60UL)

#define ETH_GPIO_WAKE					(0x64UL)
	#define     ETH_GPIO_WAKE_PHY_LINKUP_EN_ (0x80000000UL)
    	#define     ETH_GPIO_WAKE_GPIOPOL7_      (0x00800000UL)
    	#define     ETH_GPIO_WAKE_GPIOPOL0_      (0x00010000UL)
    	#define     ETH_GPIO_WAKE_GPIOWK7_       (0x00000080UL)
    	#define     ETH_GPIO_WAKE_GPIOWK0_       (0x00000001UL)


#define ETH_INT_EP_CTL					(0x68UL)
	#define ETH_INT_EP_CTL_RX_FIFO_			(0x00040000UL)
	#define	ETH_INT_EP_CTL_TX_STOP_			(0x00020000UL)
	#define	ETH_INT_EP_CTL_RX_STOP_			(0x00010000UL)
	#define	ETH_INT_EP_CTL_PHY_INT_			(0x00008000UL)
	#define	ETH_INT_EP_CTL_TXE_				(0x00004000UL)
	#define	ETH_INT_EP_CTL_TDFU_			(0x00002000UL)
	#define	ETH_INT_EP_CTL_TDFO_			(0x00001000UL)
	#define	ETH_INT_EP_CTL_RXDF_			(0x00000800UL)
	#define	ETH_INT_EP_CTL_GPIOS_			(0x000007FFUL)

#define ETH_BULK_IN_DLY 				(0x6CUL)
	#define ETH_BULK_IN_DLY_DEFAULT 		(0x00002000UL)

// MAC CSRs
#define MAC_CR						(0x100UL)
	#define MAC_CR_RXALL_					(0x80000000UL)
	#define MAC_CR_HBDIS_					(0x10000000UL)
	#define MAC_CR_PS_						(0x08000000UL)
	#define MAC_CR_RCVOWN_					(0x00800000UL)
	#define MAC_CR_LOOPBK_					(0x00200000UL)
	#define MAC_CR_FDPX_					(0x00100000UL)
	#define MAC_CR_MCPAS_					(0x00080000UL)
	#define MAC_CR_PRMS_					(0x00040000UL)
	#define MAC_CR_INVFILT_					(0x00020000UL)
	#define MAC_CR_PASSBAD_					(0x00010000UL)
	#define MAC_CR_HFILT_					(0x00008000UL)
	#define MAC_CR_HPFILT_					(0x00002000UL)
	#define MAC_CR_LCOLL_					(0x00001000UL)
	#define MAC_CR_BCAST_					(0x00000800UL)
	#define MAC_CR_DISRTY_					(0x00000400UL)
	#define MAC_CR_PADSTR_					(0x00000100UL)
	#define MAC_CR_BOLMT_MASK				(0x000000C0UL)
	#define MAC_CR_DFCHK_					(0x00000020UL)
	#define MAC_CR_TXEN_					(0x00000008UL)
	#define MAC_CR_RXEN_					(0x00000004UL)
#define ADDRH						(0x104UL)
#define ADDRL						(0x108UL)
#define HASHH						(0x10CUL)
#define HASHL						(0x110UL)

#define MII_ADDR					(0x114UL)
	#define MII_WRITE_						(0x02UL)
	#define MII_BUSY_						(0x01UL)

#define MII_DATA					(0x118UL)
#define FLOW						(0x11CUL)
	#define FLOW_FCPT_						(0xFFFF0000UL)
	#define FLOW_FCPASS_					(0x00000004UL)
	#define FLOW_FCEN_						(0x00000002UL)
	#define FLOW_FCBSY_ 					(0x00000001UL)

#define VLAN1						(0x120UL)
#define VLAN2						(0x124UL)
#define WUFF						(0x128UL)
	#define WUFF_M0_00						(0x00)
	#define WUFF_M0_01						(0x01)
	#define WUFF_M0_02						(0x02)
	#define WUFF_M0_03						(0x03)
	#define WUFF_M1_00						(0x04)
	#define WUFF_M1_01						(0x05)
	#define WUFF_M1_02						(0x06)
	#define WUFF_M1_03						(0x07)
	#define WUFF_M2_00						(0x08)
	#define WUFF_M2_01						(0x09)
	#define WUFF_M2_02						(0x0a)
	#define WUFF_M2_03						(0x0b)
	#define WUFF_M3_00						(0x0c)
	#define WUFF_M3_01						(0x0d)
	#define WUFF_M3_02						(0x0e)
	#define WUFF_M3_03						(0x0f)
	#define WUFF_M4_00						(0x10)
	#define WUFF_M4_01						(0x11)
	#define WUFF_M4_02						(0x12)
	#define WUFF_M4_03						(0x13)
	#define WUFF_M5_00						(0x14)
	#define WUFF_M5_01						(0x15)
	#define WUFF_M5_02						(0x16)
	#define WUFF_M5_03						(0x17)
	#define WUFF_M6_00						(0x18)
	#define WUFF_M6_01						(0x19)
	#define WUFF_M6_02						(0x1a)
	#define WUFF_M6_03						(0x1b)
	#define WUFF_M7_00						(0x1c)
	#define WUFF_M7_01						(0x1d)
	#define WUFF_M7_02						(0x1e)
	#define WUFF_M7_03						(0x1f)

	#define WUFF_4_CMD						(0x10)
	#define WUFF_4_OFF						(0x11)
	#define WUFF_4_CRC_01					(0x12)
	#define WUFF_4_CRC_23					(0x13)

	#define WUFF_8_CMD03					(0x20)
	#define WUFF_8_CMD47					(0x21)
	#define WUFF_8_OFF03					(0x22)
	#define WUFF_8_OFF47					(0x23)
	#define WUFF_8_CRC_01					(0x24)
	#define WUFF_8_CRC_23					(0x25)
	#define WUFF_8_CRC_45					(0x26)
	#define WUFF_8_CRC_67					(0x27)

	#define WUFF_CMD_MCAST_					(0x00000008UL)
	#define WUFF_CMD_ANY_					(0x00000004UL)
	#define WUFF_CMD_EN_					(0x00000001UL)

#define WUCSR                       (0x12CUL)
    #define WUCSR_WFF_PTR_RST   (0x80000000UL)
    #define WUCSR_GUE_          (0x00000200UL)
    #define WUCSR_PFDA_FR_      (0x00000080UL)  // LAN9500A only
    #define WUCSR_WUFR_         (0x00000040UL)
    #define WUCSR_MPR_          (0x00000020UL)
    #define WUCSR_BCAST_FR_     (0x00000010UL)  // LAN9500A only
    #define WUCSR_PFDA_EN_      (0x00000008UL)  // LAN9500A only
    #define WUCSR_WAKE_EN_      (0x00000004UL)
    #define WUCSR_MPEN_         (0x00000002UL)
    #define WUCSR_BCAST_EN_     (0x00000001UL)  // LAN9500A only

#define ETH_COE_CR					(0x130UL)
	#define Tx_COE_EN_						(0x00010000UL)
	#define Rx_COE_MODE_					(0x00000002UL)
	#define Rx_COE_EN_						(0x00000001UL)

////////////////////////////////////
// PHY Definitions
////////////////////////////////////
#define LAN_PHY_ADDR		(1UL)

#define PHY_BMC_REG 				0x0
	#define PHY_RESET					BIT_15
	#define PHY_LOOPBACK				BIT_14
	#define PHY_SPEED_SELECT_100		BIT_13
	#define PHY_AN_ENABLE				BIT_12
	#define PHY_POWER_DOWN				BIT_11
	#define PHY_ISOLATE 				BIT_10
	#define PHY_RESTART_AN				BIT_9
	#define PHY_FULL_DUPLEX 			BIT_8

#define PHY_BMS_REG 				0x1 	/* Basic Mode Status Register */
	#define PHY_AN_COMPLETE 			BIT_5
	#define	PHY_REMOTE_FAULT			BIT_4
	#define PHY_LINK					BIT_2

#define PHY_ID1_REG 				0x2 	/* Phy Identifier register 1 */

#define PHY_ID2_REG 				0x3 	/* Phy Identifier register 2 */
#define PHY_OUI_FOX					0x0001F0

#define PHY_ANA_REG					0x4
	#define PHY_ANA_NEXT_PAGE			BIT_15
	#define PHY_ANA_REMOTE_FAULT		BIT_13
	#define PHY_ANA_PAUSE_ASM_DIR		BIT_11
	#define PHY_ANA_PAUSE				BIT_10
	#define PHY_ANA_100BASE_T4			BIT_9
	#define PHY_ANA_100BASE_TX_FD		BIT_8
	#define PHY_ANA_100BASE_TX_HD		BIT_7
	#define PHY_ANA_10BASE_T_FD			BIT_6
	#define PHY_ANA_10BASE_T_HD			BIT_5
	#define PHY_ANA_SELECTOR_FIELD		0x001F
	#define PHY_ANA_ALL_SUPPORTED_MEDIA	BIT_8_7_6_5
	#define PHY_ANA_PAUSE_MASK			BIT_11_10

#define PHY_ANLPA_REG				0x5 
	#define PHY_100_BASE_T4 			BIT_9
	#define PHY_100_BASE_TX_FD			BIT_8
	#define PHY_100_BASE_TX_HD			BIT_7
	#define PHY_10_BASE_T_FD			BIT_6
	#define PHY_10_BASE_T_HD			BIT_5

#define PHY_ANE_REG					0x6 	/* Auto-Negotiation Expansion Reg */
	#define PHY_LP_AN_ABLE				BIT_0

#define PHY_THERA_SIL_REV           0x10
#define PHY_THERA_SIL_REV_TX_NLP_EN_ BIT_15
#define PHY_THERA_SIL_REV_TX_NLP_I_MSK_  (BIT_14 |BIT_13)
#define PHY_THERA_SIL_REV_TX_NLP_I_1S	0
#define PHY_THERA_SIL_REV_TX_NLP_I_768_ BIT_13
#define PHY_THERA_SIL_REV_TX_NLP_I_512_ BIT_14
#define PHY_THERA_SIL_REV_TX_NLP_I_256_ (BIT_14 | BIT_13)
#define PHY_THERA_SIL_REV_RX_1_NLP_     BIT_12
#define PHY_THERA_SIL_REV_AMDIX_XTD_ BIT_0

#define EDPD_CFG_DEFAULT	(PHY_THERA_SIL_REV_TX_NLP_EN_ | PHY_THERA_SIL_REV_TX_NLP_I_768_ | PHY_THERA_SIL_REV_RX_1_NLP_)

#define PHY_THERA_MODE_CTL_STS		0x11
	#define PHY_THERA_MODE_CTL_STS_EPD_	BIT_13
	#define PHY_THERA_MODE_CTL_STS_ED_	BIT_1

#define PHY_THERA_SPECIAL_MODES     0x12
    
#define PHY_THERA_TSTCNTL           0x14
   #define PHY_THERA_TSTCNTL_READ	(0x8000U)
	#define PHY_THERA_TSTCNTL_WRITE	(0x4000U)
	#define PHY_THERA_TSTCNTL_TEST_MODE	(0x0400U)
	#define PHY_THERA_TSTCNTL_READ_MASK	(0x02E0U)
	#define PHY_THERA_TSTCNTL_WRITE_MASK	(0x001FU)
    
    
#define PHY_THERA_TSTREAD1          0x15
#define PHY_THERA_TSTREAD2          0x16
#define PHY_THERA_TSTWRITE          0x17

#define PHY_THERA_CTL_STS_IND		0x1B
#define PHY_THERA_SP_IN_TEST		0x1C

#define PHY_THERA_INT_SOURCE		0x1D
	#define PHY_THERA_INT_ENERGY_ON		BIT_7
	#define PHY_THERA_INT_NWAY_DONE		BIT_6
	#define PHY_THERA_INT_REMOTE_FAULT	BIT_5
	#define PHY_THERA_INT_LINK_DOWN		BIT_4
	#define PHY_THERA_INT_NWAY_LP_ACK	BIT_3
	#define PHY_THERA_INT_PARALLEL_DET	BIT_2
	#define PHY_THERA_INT_NWAY_PAGE_RX	BIT_1
	#define PHY_THERA_INT_RESERVED		BIT_0

#define PHY_THERA_INT_MASK			0x1E

#define PHY_QSI_INT_SOURCE_REG		0x1D	/* Interrupt Source Register for QSI */
	#define PHY_QSI_AN_COMPLETE_INT 	BIT_6
	#define	PHY_QSI_REM_FAULT_INT		BIT_5
	#define	PHY_QSI_LINK_DOWN_INT		BIT_4
	#define	PHY_QSI_AN_LP_ACK_INT		BIT_3
	#define	PHY_QSI_PAR_DET_INT 		BIT_2
	#define	PHY_QSI_AN_PAGE_RCVD_INT	BIT_1
	#define	PHY_QSI_RCV_ERR_CNT_INT 	BIT_0

#define PHY_QSI_INT_MASK_REG		0x1E	/* Interrupt Mask Register for QSI */
	#define PHY_QSI_INT_MODE			BIT_15

/******************************************************************************/
/* Bit Mask definitions 													  */
/******************************************************************************/
#define BIT_NONE					0x00000000
#define BIT_0						0x0001
#define BIT_1						0x0002
#define BIT_2						0x0004
#define BIT_3						0x0008
#define BIT_4						0x0010
#define BIT_5						0x0020
#define BIT_6						0x0040
#define BIT_7						0x0080
#define BIT_8						0x0100
#define BIT_9						0x0200
#define BIT_10						0x0400
#define BIT_11						0x0800
#define BIT_12						0x1000
#define BIT_13						0x2000
#define BIT_14						0x4000
#define BIT_15						0x8000
#define BIT_16						0x10000
#define BIT_17						0x20000
#define BIT_18						0x40000
#define BIT_19						0x80000
#define BIT_20						0x100000
#define BIT_21						0x200000
#define BIT_22						0x400000
#define BIT_23						0x800000
#define BIT_24						0x1000000
#define BIT_25						0x2000000
#define BIT_26						0x4000000
#define BIT_27						0x8000000
#define BIT_28						0x10000000
#define BIT_29						0x20000000
#define BIT_30						0x40000000
#define BIT_31						0x80000000

#define BIT_1_0 					0x0003
#define BIT_2_1 					0x0006
#define BIT_9_8 					0x0300
#define BIT_13_12					0x3000
#define BIT_10_9_8_7				0x0780
#define BIT_11_10_9_8				0x0f00
#define BIT_11_10					0x0C00
#define BIT_7__0					0x00ff
#define BIT_7__3					0x00f8
#define BIT_8__4					0x01f0
#define BIT_8_7_6_5					0x01E0
#define BIT_11__8					0x0f00
#define BIT_13__9					0x3e00
#define BIT_14__4					0x7ff0
#define BIT_14__8					0x7f00
#define BIT_14__10					0x7c00
#define BIT_23__16					0x00ff0000
#define BIT_31__16					0xffff0000

// Vendor Requests
#define		USB_VENDOR_REQUEST_WRITE_REGISTER	0xA0
#define		USB_VENDOR_REQUEST_READ_REGISTER	0xA1
#define		USB_VENDOR_REQUEST_GET_STATS		0xA2

// Stats block structures
typedef struct _LAN9500_RX_STATS {
	DWORD	RxGoodFrames;
	DWORD	RxCrcErrors;
	DWORD	RxRuntFrameErrors;
	DWORD	RxAlignmentErrors;
	DWORD	RxFrameTooLongError;
	DWORD	RxLaterCollisionError;
	DWORD	RxBadFrames;
	DWORD	RxFifoDroppedFrames;
    DWORD   EeeRxLpiTransitions;
    DWORD   EeeRxLpiTime;
}LAN9500_RX_STATS, *PLAN9500_RX_STATS;

typedef struct _LAN9500_TX_STATS {
	DWORD	TxGoodFrames;
	DWORD	TxPauseFrames;
	DWORD	TxSingleCollisions;
	DWORD	TxMultipleCollisions;
	DWORD	TxExcessiveCollisionErrors;
	DWORD	TxLateCollisionErrors;
	DWORD	TxBufferUnderrunErrors;
	DWORD	TxExcessiveDeferralErrors;
	DWORD	TxCarrierErrors;
	DWORD	TxBadFrames;
    DWORD   EeeTxLpiTransitions;
    DWORD   EeeTxLpiTime;
} LAN9500_TX_STATS, *PLAN9500_TX_STATS;


// Interrupt Endpoint status word bitfields
#define		INT_ENP_MACRTO_						(0x00080000UL)
#define		INT_ENP_RFHF_						(0x00040000UL)
#define		INT_ENP_TX_STOP_					ETH_INT_STS_TX_STOP_
#define		INT_ENP_RX_STOP_					ETH_INT_STS_RX_STOP_
#define		INT_ENP_PHY_INT_					ETH_INT_STS_PHY_INT_
#define		INT_ENP_TXE_						ETH_INT_STS_TXE_	
#define		INT_ENP_TDFU_						ETH_INT_STS_TDFU_	
#define		INT_ENP_TDFO_						ETH_INT_STS_TDFO_	
#define		INT_ENP_RXDF_						ETH_INT_STS_RXDF_	
#define		INT_ENP_GPIOS_						ETH_INT_STS_GPIOS_	

/******************************************************************************/
/* Wake on LAN pattern structure and definitions							  */
/******************************************************************************/

#define MAX_OFFSET_OF_WOL_PATTERN	255
#define NUMBER_OF_WOL_PATTERNS_4	4		// Assumed <= 255
#define NUMBER_OF_WOL_PATTERNS_8	8		// Assumed <= 255
#define SIZE_OF_WOL_PATTERN			128
#define SIZE_OF_WOL_PATTERN_IN_DW	4
#define SIZE_OF_WOL_SAMPLE			2048	// Enough to hold any Ethernet frame.

typedef struct{
	DWORD	Enabled;
	BYTE	HeadPattern;
	BYTE	*pWOLSample;
	DWORD	WOLSampleSize;
	BYTE	*pWOLMask;
	DWORD	WOLMaskSize;
	WORD	FirstEnabledByte;
	WORD	LastEnabledByte;
} LM_WOL_SAMPLE_STRUCT, *PLM_WOL_SAMPLE_STRUCT;

typedef UCHAR	NIC_MAC_ADDRESS[6];

#define 	ETH_ADDR_SIZE				6
#define 	ETH_HEADER_SIZE 			14
#define 	ETH_MAX_DATA_SIZE			1500
#define 	ETH_MAX_PACKET_SIZE 		(ETH_HEADER_SIZE + ETH_MAX_DATA_SIZE+4)
#define 	ETH_MIN_PACKET_SIZE 		60
#define 	ETH_MAX_FRAME_SIZE			(ETH_MAX_PACKET_SIZE + 8UL) 

#define ETH_IS_LOCALLY_ADMINISTERED(Address) \
	(BOOLEAN)(((PUCHAR)(Address))[0] & ((UCHAR)0x02))

#define NIC_MAX_MCAST_LIST				128

#define NIC_MIN_VAL_FOR_MAX_BUSY_SENDS	1
#define NIC_MIN_VAL_FOR_MAX_BUSY_RECVS	1
#define NIC_MIN_VAL_FOR_MAX_BUSY_INTS	1	

#define NIC_MAX_BUSY_SENDS_DEFAULT		2
#define NIC_MAX_BUSY_RECVS_DEFAULT		2
#define NIC_MAX_BUSY_INTS_DEFAULT		2	

#define NIC_MIN_VAL_FOR_RECVS_LOW_WM	1
#define NIC_MIN_VAL_FOR_INTS_LOW_WM		1	

#define NIC_MAX_LOOKAHEAD				ETH_MAX_DATA_SIZE
#define NIC_BUFFER_SIZE 				(2 * 1024)
#define INT_BUFFER_SIZE 				4UL	// (Int Status is 1 dword)
#define NIC_ADAPTER_NAME_SIZE			128

#define BUFFER_SIZE_4KB						(4 * 1024)
#define BUFFER_SIZE_6KB						(6 * 1024)
#define BUFFER_SIZE_8256_BYTES				(6 * 1024 + 33 * 64)
#define BUFFER_SIZE_16KB					(16 * 1024)
#define BUFFER_SIZE_18_5KB					(16 * 1024 + 5 * 512)

#define BUFFER_SIZE_8KB					(8 * 1024)

//
// Buffer size passed in NdisMQueryAdapterResources 						   
// We should only need three adapter resources (IO, interrupt and memory),
// Some devices get extra resources, so have room for 10 resources 
//
#define NIC_RESOURCE_BUF_SIZE			(sizeof(NDIS_RESOURCE_LIST) + \
										(10*sizeof(CM_PARTIAL_RESOURCE_DESCRIPTOR)))

#define NIC_SUPPORTED_FILTERS ( \
				NDIS_PACKET_TYPE_DIRECTED	| \
				NDIS_PACKET_TYPE_MULTICAST	| \
				NDIS_PACKET_TYPE_BROADCAST	| \
				NDIS_PACKET_TYPE_PROMISCUOUS | \
				NDIS_PACKET_TYPE_ALL_MULTICAST)

#define fMP_INIT_IN_PROGRESS					0x00000001
#define fMP_RESET_IN_PROGRESS					0x00000002
#define fMP_DISCONNECTED						0x00000004
#define fMP_HALT_IN_PROGRESS					0x00000008
#define fMP_SURPRISE_REMOVED					0x00000010
#define fMP_SEND_SIDE_RESOURCE_ALLOCATED		0x00000020
#define fMP_RECV_SIDE_RESOURCE_ALLOCATED		0x00000040
#define fMP_INT_RESOURCE_ALLOCATED				0x00000080
#define fMP_POST_INTS							0x00000100
#define fMP_POST_READS							0x00001000
#define fMP_IGNORE_READS						0x00002000
#define fMP_POST_WRITES							0x00004000
#define fMP_PERIODIC_LINK_CHECK					0x00008000
#define fMP_LINK_CHECK_IN_PROGRESS				0x00010000
#define fMP_REQUEST_PENDING						0x00020000
#define fMP_USB_RESET_IN_PROGRESS				0x00040000
#define fMP_NEED_USB_RESET						0x00080000
#define fMP_NEED_FULL_RESET						0x00100000
#define fMP_IOCTL_REGISTERED					0x00200000
#define fMP_DRVCMD_IN_PROGRESS					0x00400000
#define	fMP_DEVICE_NOT_ATTACHED					0x00800000




typedef struct	_SHOW_REG	{
	WCHAR	wszName[50];
	USHORT	 dwOffset;
}	SHOW_REG;

static const SHOW_REG	sysCsr[] = {
					  {L"ETH_ID_REV 	 ", ETH_ID_REV},
						 {L"ETH_FPGA_REV	", ETH_FPGA_REV},
						 {L"ETH_INT_STS 	", ETH_INT_STS},
						 {L"ETH_RX_CFG		", ETH_RX_CFG},
						 {L"ETH_TX_CFG		", ETH_TX_CFG},
						 {L"ETH_HW_CFG		", ETH_HW_CFG},
						 {L"ETH_RX_FIFO_INF ", ETH_RX_FIFO_INF},
						 {L"ETH_TX_FIFO_INF ", ETH_TX_FIFO_INF},
						 {L"ETH_PM_CTRL 	", ETH_PM_CTRL},
						 {L"ETH_LED_GPIO_CFG", ETH_LED_GPIO_CFG},
						 {L"ETH_GPIO_CFG	", ETH_GPIO_CFG},
						 {L"ETH_AFC_CFG 	", ETH_AFC_CFG},
						 {L"ETH_E2P_CMD 	", ETH_E2P_CMD},
						 {L"ETH_BURST_CAP	", ETH_BURST_CAP},
						 {L"ETH_STRAP_DEBUG ", ETH_STRAP_DEBUG},
						 {L"ETH_DP_SEL		", ETH_DP_SEL},
						 {L"ETH_DP_CMD		", ETH_DP_CMD},
						 {L"ETH_DP_ADDR 	", ETH_DP_ADDR},
						 {L"ETH_DP_DATA0	", ETH_DP_DATA0},
						 {L"ETH_DP_DATA1	", ETH_DP_DATA1},
						 {L"ETH_GPIO_WAKE	", ETH_GPIO_WAKE},
						 {L"ETH_INT_EP_CTL	", ETH_INT_EP_CTL},
						 {L"ETH_BULK_IN_DLY ", ETH_BULK_IN_DLY}
						 };

static const SHOW_REG macCsr[] = {
					  {L"MAC_CR    ", MAC_CR},
						 {L"ADDRH	  ", ADDRH},
						 {L"ADDRL	  ", ADDRL},
						 {L"HASHH	  ", HASHH},
						 {L"HASHL	  ", HASHL},
						 {L"MII_ADDR  ", MII_ADDR},
						 {L"MII_DATA  ", MII_DATA},
						 {L"FLOW	  ", FLOW},
						 {L"VLAN1	  ", VLAN1},
						 {L"VLAN2	  ", VLAN2},
						 {L"WUFF	  ", WUFF},
						 {L"WUCSR	  ", WUCSR},
						 {L"ETH_COE_CR", ETH_COE_CR}
			};

static const SHOW_REG phyCsr[] ={
					  {L"PHY_BMC_REG			", PHY_BMC_REG},
						 {L"PHY_BMS_REG 		   ", PHY_BMS_REG},
						 {L"PHY_ID1_REG 		   ", PHY_ID1_REG},
						 {L"PHY_ID2_REG 		   ", PHY_ID2_REG},
						 {L"PHY_ANA_REG 		   ", PHY_ANA_REG},
						 {L"PHY_ANLPA_REG		   ", PHY_ANLPA_REG},
						 {L"PHY_ANE_REG 		   ", PHY_ANE_REG},
						 {L"PHY_THERA_MODE_CTL_STS ", PHY_THERA_MODE_CTL_STS},
						 {L"PHY_THERA_INT_SOURCE   ", PHY_THERA_INT_SOURCE},
						 {L"PHY_THERA_INT_MASK	   ", PHY_THERA_INT_MASK},
			};



#ifndef min
#define    min(_a, _b)		(((_a) < (_b)) ? (_a) : (_b))
#endif

#ifndef max
#define    max(_a, _b)		(((_a) > (_b)) ? (_a) : (_b))
#endif

//--------------------------------------
// Utility macros		 
//--------------------------------------

#define MP_SET_FLAG(_M, _F) 			((_M)->Flags |= (_F))
#define MP_CLEAR_FLAG(_M, _F)			 ((_M)->Flags &= ~(_F))
#define MP_TEST_FLAG(_M, _F)			(((_M)->Flags & (_F)) != 0)
#define MP_TEST_FLAGS(_M, _F)			 (((_M)->Flags & (_F)) == (_F))

#define MP_IS_READY(_M) 	   (((_M)->Flags & \
								 (fMP_RESET_IN_PROGRESS | fMP_HALT_IN_PROGRESS | fMP_INIT_IN_PROGRESS | fMP_SURPRISE_REMOVED | fMP_DEVICE_NOT_ATTACHED)) == 0) 

#define MP_IS_RESET_PENDING(_M) 	   (((_M)->Flags & \
										 (fMP_NEED_USB_RESET | fMP_NEED_FULL_RESET)) != 0) 

#define MP_IS_READY_AND_LINKED_UP(_M)		 (((_M)->Flags & \
											   (fMP_DISCONNECTED | fMP_RESET_IN_PROGRESS | fMP_HALT_IN_PROGRESS | fMP_INIT_IN_PROGRESS | fMP_SURPRISE_REMOVED | fMP_DEVICE_NOT_ATTACHED)) == 0) 

#define MP_INC_REF(_A)				NdisInterlockedIncrement(&(_A)->RefCount)

#define MP_DEC_REF(_A) {\
							NdisInterlockedDecrement(&(_A)->RefCount);\
							ASSERT(_A->RefCount >= 0);\
							if((_A)->RefCount == 0){\
								NdisSetEvent(&(_A)->HaltEvent);\
							}\
						}

#define MP_GET_REF(_A)				((_A)->RefCount)
 
#include <pshpack1.h>

typedef struct _ETH_HEADER
{
	UCHAR		DstAddr[ETH_LENGTH_OF_ADDRESS];
	UCHAR		SrcAddr[ETH_LENGTH_OF_ADDRESS];
	USHORT		EthType;
} ETH_HEADER, *PETH_HEADER;

#include <poppack.h>

//
// This structure is used to send requests 
//
typedef struct _WDM_NDIS_REQUEST
{
	NDIS_WORK_ITEM		WorkItem; // must be first in the list
	NDIS_OID			Oid;
	NDIS_REQUEST_TYPE	RequestType;
	PVOID				InformationBuffer;
	ULONG				InformationBufferLength;
	PULONG				BytesReadOrWritten;
	PULONG				BytesNeeded;

} WDM_NDIS_REQUEST, *PWDM_NDIS_REQUEST;


// TCB (Transmit Control Block)
typedef struct _TCB
{
	LIST_ENTRY				List; // This must be the first entry
	LONG					Ref;
	PVOID					Adapter;
	PNDIS_PACKET			OrgSendPacket;
	PUCHAR					pData;		// Pointer to start of memory buffer
	ULONG					ulSize;
	BOOLEAN 				bFull;
	ULONG					nPktCount;
	DWORD					dwErr;
	USB_ERROR				dwUsbErr;
	DWORD					dwTransferLength;
	HANDLE					SyncEvent;
} TCB, *PTCB;

// RCB (Receive Control Block)
typedef struct _RCB
{
	LIST_ENTRY				List; // This must be the first entry
	LONG					Ref;
	PVOID					Adapter;
	#ifdef DEBUG
	ULONG					tag;
	#endif
	ULONG					seq;
	PNDIS_BUFFER			Buffer;
	PNDIS_PACKET			Packet;
	PUCHAR					pData;		  
	ULONG					ulSize;
	ULONG					ulCopied;
	USB_ERROR				dwUsbErr;
	DWORD						dwErr;
	DWORD					dwTransferLength;
	HANDLE					SyncEvent;
	ULONG					BulkInRCB;
} RCB, *PRCB;

// ICB (Interrupt Control Block)
typedef struct _ICB
{
	LIST_ENTRY				List; // This must be the first entry
	LONG					Ref;
	PVOID					Adapter;
	PUCHAR					pData;		  
	ULONG					ulSize;
	UCHAR					Data[INT_BUFFER_SIZE];
	USB_ERROR				dwUsbErr;
	DWORD						dwErr;
	DWORD					dwTransferLength;
	HANDLE					SyncEvent;
} ICB, *PICB;


#define NDIS_STATUS_TO_NT_STATUS(_NdisStatus, _pNtStatus)							\
{																					\
																				\
	if (((NDIS_STATUS_SUCCESS == (_NdisStatus)) ||									\
		(NDIS_STATUS_PENDING == (_NdisStatus)) ||									\
		(NDIS_STATUS_BUFFER_OVERFLOW == (_NdisStatus)) ||							\
		(NDIS_STATUS_FAILURE == (_NdisStatus)) ||									\
		(NDIS_STATUS_RESOURCES == (_NdisStatus)) || 								\
		(NDIS_STATUS_NOT_SUPPORTED == (_NdisStatus))))								\
	{																				\
		*(_pNtStatus) = (NTSTATUS)(_NdisStatus);									\
	}																				\
	else if (NDIS_STATUS_BUFFER_TOO_SHORT == (_NdisStatus)) 						\
	{																				\
																				 \
		*(_pNtStatus) = STATUS_BUFFER_TOO_SMALL;									\
	}																				\
	else if (NDIS_STATUS_INVALID_LENGTH == (_NdisStatus))							\
	{																				\
		*(_pNtStatus) = STATUS_INVALID_BUFFER_SIZE; 								\
	}																				\
	else if (NDIS_STATUS_INVALID_DATA == (_NdisStatus)) 							\
	{																				\
		*(_pNtStatus) = STATUS_INVALID_PARAMETER;									\
	}																				\
	else if (NDIS_STATUS_ADAPTER_NOT_FOUND == (_NdisStatus))						\
	{																				\
		*(_pNtStatus) = STATUS_NO_MORE_ENTRIES; 									\
	}																				\
	else if (NDIS_STATUS_ADAPTER_NOT_READY == (_NdisStatus))						\
	{																				\
		*(_pNtStatus) = STATUS_DEVICE_NOT_READY;									\
	}																				\
	else																			\
	{																				\
		*(_pNtStatus) = STATUS_UNSUCCESSFUL;										\
	}																				\
}

#define NT_STATUS_TO_NDIS_STATUS(_NtStatus, _pNdisStatus)							\
{																					\
																			   \
	if (((STATUS_SUCCESS == (_NtStatus)) || 								 \
		(STATUS_PENDING == (_NtStatus)) ||									 \
		(STATUS_BUFFER_OVERFLOW == (_NtStatus)) ||							 \
		(STATUS_UNSUCCESSFUL == (_NtStatus)) || 								  \
		(STATUS_INSUFFICIENT_RESOURCES == (_NtStatus)) ||								  \
		(STATUS_NOT_SUPPORTED == (_NtStatus)))) 							 \
	{																				\
		*(_pNdisStatus) = (NDIS_STATUS)(_NtStatus); 								   \
	}																				\
	else if (STATUS_BUFFER_TOO_SMALL == (_NtStatus))						 \
	{																				\
																				 \
		*(_pNdisStatus) = NDIS_STATUS_BUFFER_TOO_SHORT; 						   \
	}																				\
	else if (STATUS_INVALID_BUFFER_SIZE == (_NtStatus)) 						  \
	{																				\
		*(_pNdisStatus) = NDIS_STATUS_INVALID_LENGTH;								  \
	}																				\
	else if (STATUS_INVALID_PARAMETER == (_NtStatus))							  \
	{																				\
		*(_pNdisStatus) = NDIS_STATUS_INVALID_DATA; 								  \
	}																				\
	else if (STATUS_NO_MORE_ENTRIES == (_NtStatus)) 					   \
	{																				\
		*(_pNdisStatus) = NDIS_STATUS_ADAPTER_NOT_FOUND;									 \
	}																				\
	else if (STATUS_DEVICE_NOT_READY == (_NtStatus))						\
	{																				\
		*(_pNdisStatus) = NDIS_STATUS_ADAPTER_NOT_READY;									\
	}																				\
	else																			\
	{																				\
		*(_pNdisStatus) = NDIS_STATUS_FAILURE;										  \
	}																				\
}



#endif	  // _LAN9500_H



